// Copyright (c) 2013 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#ifndef HU_VARADIISTVAN_SCPL_IO_WIN32_LOCALSOCKET_H
#define HU_VARADIISTVAN_SCPL_IO_WIN32_LOCALSOCKET_H
//------------------------------------------------------------------------------

#include "BufferedStream.h"

#include "Overlapped.h"

#include <cstdio>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * Basically a different name for a buffered stream.
 */
class LocalSocket : public BufferedStream
{
public:
    /**
     * Create a concrete pipe path from the given user-specified name.
     *
     * @param dest the destination buffer, which should be at least
     * 256 bytes long.
     */
    static void setupPath(char* dest, const char* name);

protected:
    /**
     * Construct the local socket
     */
    LocalSocket(Waiter* waiter = 0, HANDLE handle = 0,
                size_t readingCapacity = DEFAULT_CAPACITY,
                size_t writingCapacity = DEFAULT_CAPACITY);


protected:
    /**
     * Connect the client socket to the server.
     */
    bool connect(const char* path);

    friend class LocalAcceptor;
    friend class LocalConnector;
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline void LocalSocket::setupPath(char* dest, const char* name)
{
    snprintf(dest, 256, "\\\\.\\pipe\\%s", name);
}

//------------------------------------------------------------------------------

inline LocalSocket::LocalSocket(Waiter* waiter, HANDLE handle,
                                size_t readingCapacity, size_t writingCapacity):
    BufferedStream(waiter, handle, readingCapacity, writingCapacity)
{
}

//------------------------------------------------------------------------------

inline bool LocalSocket::connect(const char* path)
{
    handle = CreateFile(path, GENERIC_READ|GENERIC_WRITE,
                        0, 0, OPEN_EXISTING,
                        FILE_FLAG_NO_BUFFERING | FILE_FLAG_OVERLAPPED, 0);

    if (handle==INVALID_HANDLE_VALUE) {
        setErrorCode(GetLastError());
    }

    return handle!=INVALID_HANDLE_VALUE;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------
#endif // HU_VARADIISTVAN_SCPL_IO_WIN32_LOCALSOCKET_H

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
